//
// cen64.h: CEN64 entry point.
//
// CEN64: Cycle-Accurate Nintendo 64 Emulator.
// Copyright (C) 2015, Tyler J. Stachecki.
//
// This file is subject to the terms and conditions defined in
// 'LICENSE', which is part of this source code package.
//

#ifndef __cen64_h__
#define __cen64_h__
#include "common.h"

cen64_cold int cen64_main(int argc, const char **argv);

#endif

