//
// arch/arm/fpu/add_64.h
//
// This file is subject to the terms and conditions defined in
// 'LICENSE', which is part of this source code package.
//

static inline void fpu_add_64(
  const uint64_t *fs, const uint64_t *ft, uint64_t *fd) {
}

