//
// os/main.h
//
// Convenience functions for managing rendering windows.
//
// This file is subject to the terms and conditions defined in
// 'LICENSE', which is part of this source code package.
//

#ifndef __os_main_h__
#define __os_main_h__
#include "device/device.h"
#include "device/options.h"
#include "os/gl_window.h"
#include "rom_file.h"

#endif

