//
// common/reciprocal.h: RSP reciprocal ROM contents.
//
// CEN64: Cycle-Accurate Nintendo 64 Emulator.
// Copyright (C) 2015, Tyler J. Stachecki.
//
// This file is subject to the terms and conditions defined in
// 'LICENSE', which is part of this source code package.
//

#ifndef __common_reciprocal_h__
#define __common_reciprocal_h__
#include "common.h"

extern const uint16_t rsp_reciprocal_rom[1024];

#endif

