//
// arch/x86_64/rsp/gcc/defs.h
//
// CEN64: Cycle-Accurate Nintendo 64 Emulator.
// Copyright (C) 2015, Tyler J. Stachecki.
//
// This file is subject to the terms and conditions defined in
// 'LICENSE', which is part of this source code package.
//

.set acc_lo, %xmm8
.set acc_md, %xmm9
.set acc_hi, %xmm10

.set vcc_lo, %xmm11
.set vcc_hi, %xmm12
.set vco_lo, %xmm13
.set vco_hi, %xmm14
.set vce   , %xmm15

