//
// arch/arm/fpu/add_32.h
//
// This file is subject to the terms and conditions defined in
// 'LICENSE', which is part of this source code package.
//

static inline void fpu_add_32(
  const uint32_t *fs, const uint32_t *ft, uint32_t *fd) {
}

